clear all;
%% setup path
if 1
%addpath('/afs/cs.stanford.edu/u/amaas/scratch/libsvm-mat-2.91-1');
addpath('/afs/cs.stanford.edu/u/amaas/scratch/liblinear-1.6/matlab');

end;
%% demo parameters
% list of categories to evaluate on
catList = {'dvd','books','kitchen','electronics'};
baseStr = '../tmp/amzNew';
% param for whether to weight once or weight excluding test example
% (setting this non-zero means you compute weights on test set)
weightOnce = 0;
%% build concatenated data matrix
docLabel = []; docBOW = [];
for c = 1 : length(catList)
    amzCat = catList{c};    
    catFile = sprintf('%s.%s.feat',baseStr,amzCat);
    [curDocLabel, curDocBOW] = libsvmread(catFile);
    % make sure sparse matrices match in size
    
    [curM,curN] = size(curDocBOW);
    [m,n] = size(docBOW);
    if curN < n
        [i,j,s] = find(curDocBOW);
        curDocBOW = sparse(i,j,s,curM,n);
    elseif curN > n
        [i,j,s] = find(docBOW);
        docBOW = sparse(i,j,s,m,curN);
    end;
        %docBOW = docBOW(:,1:90000);
    docLabel = [docLabel;curDocLabel];
    docBOW = [docBOW;curDocBOW];
end
%% randomly permute inputs necessary (stored in blocks)
rp = randperm(size(docBOW,1));
docBOW = docBOW(rp,:);
docLabel = docLabel(rp);

%% perform loocv in parallel
numExamples = size(docBOW,1);
%numExamples = 100;
accRes = [];
irTestFeatMat = [];
irTestFeatLabel = [];
parfor i = 1:numExamples
    testLabel = docLabel(i,:);
    trainInd = true(numExamples,1); trainInd(i)=false;
    if weightOnce
        %% weight features using all examples
        [ irFeatAll, wIdf ] = weightBoW( docBOW, docLabel, 1, 1, 0);
        irFeatTest = irFeatAll(i,:);
        irFeatTrain = irFeatAll(trainInd,:);
    else
        %% construct weighting on train set only
        [ irFeatTrain, wIdf ] = weightBoW( docBOW(trainInd,:), ...
            docLabel(trainInd),    1, 1, 0);
        %% apply learned weighting to test and val (does not use test set labels)
        [ irFeatTest, dummyIdf ] = weightBoW( docBOW(i,:), [], ...
            1, 1, 0, wIdf);
    end
    %% build improper data matrix to use later in loocv
    irTestFeatMat = [irTestFeatMat; irFeatTest];
    irTestFeatLabel = [irTestFeatLabel; testLabel];
end;
fprintf(1,'weighting complete. performing loocv\n');
%% now do loocv on the computed features
parfor i = 1:numExamples
    testLabel = irTestFeatLabel(i,:);
    trainInd = true(numExamples,1); trainInd(i)=false;
    %% train SVM with default params        
    curModel = train(irTestFeatLabel(trainInd), irTestFeatMat(trainInd,:));
    %% predict 
    [dummyPred,cRes] = predict(testLabel,irTestFeatMat(i,:),curModel);
    accRes = [accRes;cRes];
    %fprintf(1,'cur mean acc::  %f\n',mean(accRes(1:i)));
end
fprintf(1,'final results: %f \n', mean(accRes));

