function [ outFeat, wIdf ] = weightBoW( feat, labels, termP, invdocP, normP, wIdf )
%WEIGHTBOW IR weighting schemes for BoW features
% feat is term/doc counts (docs per row). labels are 0/1
% termP 0 - natural, 1 - binary
% invdocP 0 - none, 1 -delta t' 2 - idf
% normP 0 - none, 1 - cosine

%% get idf weights
% for test examples wIdf might be supplied
if ~exist('wIdf','var')
    % all 1's is none case
    wIdf = ones(1,size(feat,2));
    if invdocP == 1
        %% delta t' idf
        posInd = find(labels > 0);
        negInd = find(labels <= 0);
        % sum number of docs for each word
        posSum = full(sum(feat(posInd,:)>0,1));
        negSum = full(sum(feat(negInd,:)>0,1));
        %wIdf = log2((0.5+length(posInd)*negSum) ./ (0.5+length(negInd)*posSum));
        wIdf = log2((0.5+length(negInd)*posSum) ./ (0.5+length(posInd)*negSum));
    elseif invdocP == 2
         %% vanilla idf
         docSum = full(sum(feat > 0,1));
         wIdf = log2(size(feat,1) ./ (1 + docSum));
    end;
    
end;
    %% get tf weights
outFeat = feat;
if termP == 1
    outFeat = min(outFeat,1);
end;

%% weight features with idf
outFeat = bsxfun(@times,outFeat,wIdf);

%% normalize
if normP == 1
    outFeat = bsxfun(@rdivide, outFeat, sqrt(sum(outFeat.^2,2)));
end;
%% HACK set nan features to 0
outFeat(isnan(outFeat)) = 0;
