# convert blitzer 2007 data to BoW

import sys, fileinput, operator
from glob import glob

class amzFeatureConverter(object):
    def __init__(self):
        #maps feature strings to indices
        self.featureDict = {}

    # load data from file already in BoW form
    # write features to stdout

    def writeAmzFeat(self, inFname, outFile):
        for revStr in fileinput.input([inFname]):
            revFeatList = []
            revLabel = "-1"
            for featStr in revStr.split():
                # <featWord>:<Count> tokens
                w = featStr.split(":")[0]
                c = featStr.split(":")[1]
                # check for review label
                if w == "#label#":
                    if c == "positive":
                        revLabel = '1'
                    else:
                        revLabel = '0'
                    continue
                # get index of word
                if not self.featureDict.has_key(w):                    
                    self.featureDict[w] = len(self.featureDict)+1
                wKey = self.featureDict[w]
                # apped word index and its count to our list
                revFeatList.append((wKey,c))
            # sort features by index (libsvm requires)
            revFeatList = sorted(revFeatList, key=operator.itemgetter(0))
            #print revFeatList
            outFile.write("%s %s\n" % (revLabel," ".join(["%d:%s" % (wf[0],wf[1]) for wf in revFeatList])))

def unsupMain(dataPath, amzFC, outFile):
    """ 
    main function to collect unlabeled data
    Assumes the dataPath contains directories with positive.review etc files
    Creates a single libsvm file aggregating across subdirs 
    and ignoring labels
    """
    for posFile in glob("%s/*/processed.review" % dataPath):
        print >> sys.stderr, posFile
        amzFC.writeAmzFeat(posFile, outFile)
    return amzFC

if __name__ == "__main__":
    # only 2 command line args
    if len(sys.argv) < 4:
        print >> sys.stderr, "Usage: python amzToBoW.py <unsupDir> <supDir> <outputPrefix>"
    
    dataPath = sys.argv[2]
    outPrefix = sys.argv[3]
    amzFC = amzFeatureConverter()
    # now process each labeled file
    for dataDir in ['dvd','books','kitchen','electronics']:        
        posFname = dataPath+dataDir+"/positive.review"
        negFname = dataPath+dataDir+"/negative.review"
        outFname = outPrefix+dataDir+".feat"
        print >> sys.stderr, dataDir, outFname
        # load and write feat for files
        with open(outFname,"w") as curOutFile:
            amzFC.writeAmzFeat(posFname, curOutFile)
            amzFC.writeAmzFeat(negFname, curOutFile)
            
    # make single unlabeled file
    outFname = outPrefix+"unsup.feat"
    print >> sys.stderr, "unsupervised", outFname
    # load and write feat for files
    with open(outFname,"w") as curOutFile:
        amzFC = unsupMain(sys.argv[1],amzFC, curOutFile)
